#!/usr/bin/env python
"""
Copyright (c) 2012, 2013 TortoiseLabs LLC

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

This software is provided 'as is' and without any warranty, express or
implied. In no event shall the authors be liable for any damages arising
from the use of this software.
"""

import message

def encode(secret, method, iterations=1000, *args, **kwargs):
    """
    Encode a message.

    >>> secret = 'moocows'
    >>> encode(secret, 'moo', meow='cats')
    '{"params": {"meow": "cats"}, "method": "moo", "signature": "545ce308d8f3b81fc18638955571aa43f81b71007dc477d78868f7c01b754123556bd958c144db8ef3412a39cd98882207864ce83ba500c3279c40e22852da0d"}'
    """
    inner_envelope = {'method': method, 'params': kwargs}
    return message.dumps(inner_envelope, secret, iterations=iterations)

def decode(secret, signed_message, allow_unsigned=False, iterations=1000):
    """
    Return a tuple of the method and kwargs.

    >>> secret = 'moocows'
    >>> message = '{"params": {"meow": "cats"}, "method": "moo", "signature": "545ce308d8f3b81fc18638955571aa43f81b71007dc477d78868f7c01b754123556bd958c144db8ef3412a39cd98882207864ce83ba500c3279c40e22852da0d"}'
    >>> decode(secret, message)
    (u'moo', {u'meow': u'cats'})
    """
    unpack_message = message.loads(signed_message, secret, allow_unsigned, iterations)
    method = unpack_message.pop('method', None)
    params = unpack_message.pop('params', None)
    return (method, params)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
"""
Copyright (c) 2012, 2013 TortoiseLabs LLC

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

This software is provided 'as is' and without any warranty, express or
implied. In no event shall the authors be liable for any damages arising
from the use of this software.

The message module can be used to yield signed JSON structures, like so:

>>> secret = '592783ded97d505759a8b25248c3fc5b'
>>> message = { 'moo': 'cow' }
>>> dumps(message, secret)
'{"moo": "cow", "signature": "b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025"}'
"""

import json
import hashlib

from pbkdf2 import pbkdf2_hex

import copy
from collections import OrderedDict

def serializable_dict(d):
    """
    Prepare a dictionary for safe serialization to JSON.

    >>> d = {'moo': 'cow', 'cow': {'septic': 'tank', 'grass': 'greener'}}
    >>> od = serializable_dict(d)
    >>> od
    OrderedDict([('cow', OrderedDict([('grass', 'greener'), ('septic', 'tank')])), ('moo', 'cow')])
    >>> import json
    >>> json.dumps(od)
    '{"cow": {"grass": "greener", "septic": "tank"}, "moo": "cow"}'
    """
    li = []
    for item in d.items():
        if isinstance(item[1], dict):
            li.append((item[0], serializable_dict(item[1])))
        else:
            li.append((item[0], item[1]))
    return OrderedDict(sorted(li))

class Message(object):
    """
    A class object which wraps a JSON structure providing signature
    and validation services.

    An example of using the constructor:
    >>> m = Message({ 'moo': 'cow' }, '592783ded97d505759a8b25248c3fc5b')
    >>> m.payload_json()
    '{"moo": "cow"}'
    """
    def __init__(self, obj, secret, expected_hash=None, iterations=1000):
        """
        The constructor for the Message object.

        Giving it an unsigned object should yield the following kind of object:

        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b')
        >>> m.secret == '592783ded97d505759a8b25248c3fc5b'
        True
        >>> m.obj
        {'moo': 'cow'}
        >>> m.expected_hash
        """
        self.obj = obj.copy()
        self.secret = secret
        self.expected_hash = expected_hash
        self.iterations = iterations

    def payload_json(self):
        """
        Returns the inner 'payload' JSON structure without the signature attached.

        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b')
        >>> m.payload_json()
        '{"moo": "cow"}'
        """
        return json.dumps(serializable_dict(self.obj))

    def signature(self):
        """
        Calculates the signature for the message.

        The signature is just a PBKDF2 hash of the payload toasted against the secret, which is 64 bytes long.

        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b')
        >>> m.signature()
        'b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025'
        """
        signature = pbkdf2_hex(self.payload_json(), self.secret, iterations=self.iterations, keylen=64, hashfunc=hashlib.sha512)
        return signature

    def dumps(self, pretty_print=False):
        """
        Dumps the message with the signature calculated and attached.

        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b')
        >>> m.dumps()
        '{"moo": "cow", "signature": "b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025"}'
        """
        envelope = self.obj
        envelope['signature'] = self.signature()

        if pretty_print:
            return json.dumps(envelope, indent=4, separators=(',', ': '))

        return json.dumps(envelope)

    def validate(self):
        """
        Validates the message against the expected signature.  If the message is unsigned, this
        always returns True.

        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b')
        >>> m.validate()
        True
        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b', 'b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025')
        >>> m.validate()
        True
        >>> m = Message({'moo': 'cow'}, '592783ded97d505759a8b25248c3fc5b', 'abcdef123456')
        >>> m.validate()
        False
        """
        if self.expected_hash is None:
            return True

        return (self.signature() == self.expected_hash)

    def payload(self):
        """
        Returns the message payload.

        >>> structure = {'moo': 'cow'}
        >>> m = Message(structure, '592783ded97d505759a8b25248c3fc5b')
        >>> m.payload()
        {'moo': 'cow'}
        >>> m.payload() == structure
        True
        """
        return self.obj

class InvalidSignatureException(Exception):
    pass

def loads(json_data, secret, allow_unsigned=False, iterations=1000):
    """
    Unpack a JSON envelope containing an Edia RPC message.

    >>> secret = '592783ded97d505759a8b25248c3fc5b'
    >>> message = '{"moo": "cow", "signature": "b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025"}'
    >>> loads(message, secret)
    {u'moo': u'cow'}
    >>> message = '{"moo": "cow", "signature": "abcdef123456"}'
    >>> loads(message, secret)
    Traceback (most recent call last):
        ...
    InvalidSignatureException
    >>> message = '{"moo": "cow"}'
    >>> loads(message, secret)
    Traceback (most recent call last):
        ...
    InvalidSignatureException
    >>> loads(message, secret, allow_unsigned=True)
    {u'moo': u'cow'}
    """
    envelope = json.loads(json_data)

    signature = envelope.pop('signature', None)
    if signature is None and allow_unsigned is not True:
        raise InvalidSignatureException()

    message = Message(envelope, secret, signature, iterations)
    if message.validate() is not True and allow_unsigned is not True:
        raise InvalidSignatureException()

    return message.payload()

def dumps(obj, secret, pretty_print=False, iterations=1000):
    """
    Pack a dictionary into an Edia RPC message.

    >>> secret = '592783ded97d505759a8b25248c3fc5b'
    >>> message = {'moo': 'cow'}
    >>> signed_message = dumps(message, secret)
    >>> signed_message
    '{"moo": "cow", "signature": "b617fdca6b994fa3cb45ea744e3b66f6daee37e24cd1e6df50a6ff38094549a412498c5830144000d3ed451efd39934a6cdc495cb0c86c892f66162a944e6025"}'
    >>> signed_message == dumps(message, secret)
    True
    >>> unpack_message = loads(signed_message, secret)
    >>> unpack_message == message
    True
    >>> signed_message == dumps(unpack_message, secret)
    True
    """
    return Message(obj, secret, iterations=iterations).dumps(pretty_print)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
"""
Copyright (c) 2012, 2013 TortoiseLabs LLC

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

This software is provided 'as is' and without any warranty, express or
implied. In no event shall the authors be liable for any damages arising
from the use of this software.
"""

import socket
import rpc_message

class ServerProxy(object):
    def __init__(self, host, port, secret, allow_untrusted=False, iterations=1000):
        self._host = host
        self._port = port
        self._secret = secret
        self._allow_untrusted = allow_untrusted
        self._iterations = iterations

    def _read_loop(self, sock):
        data = []
        sock.settimeout(None)
        data.append(sock.recv(1024))
        sock.settimeout(0.1)

        while True:
            try:
                data.append(sock.recv(1024))
            except:
                break

        sock.settimeout(None)
        return ''.join(data)

    def _call(self, name, **kwargs):
        sock = socket.create_connection((self._host, self._port))

        envelope = rpc_message.encode(self._secret, name, **kwargs) + '\r\n'
        sock.sendall(envelope) 
        response = self._read_loop(sock)
        sock.close()

        return rpc_message.decode(self._secret, response, self._allow_untrusted, iterations=self._iterations)[1]

    def __getattr__(self, name):
        """
        Create a function which makes a RPC call.
        """
        def f(*args, **kwargs):
            return self._call(name, **kwargs)

        return f

if __name__ == '__main__':
    proxy = ServerProxy('127.0.0.1', 5959, 'moocows')
    print proxy.ping(moo='cow')



#!/usr/bin/env python
"""
Copyright (c) 2012, 2013 TortoiseLabs LLC

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

This software is provided 'as is' and without any warranty, express or
implied. In no event shall the authors be liable for any damages arising
from the use of this software.
"""

import SocketServer
import rpc_message
from message import InvalidSignatureException

class RPCRequestHandler(SocketServer.StreamRequestHandler):
    allow_reuse_address = True
    def handle(self):
        while True:
            line = self.rfile.readline()
            if not line:
                break
            response = self.server.dispatch(line)
            self.wfile.write(response)

class RPCServer(SocketServer.TCPServer, RPCRequestHandler):
    def __init__(self, tuple, secret, iterations=1000):
        SocketServer.TCPServer.__init__(self, tuple, RPCRequestHandler)
        self.funcs = {}
        self.secret = secret
        self.iterations = iterations

    def _reply(self, method, **kwargs):
        return rpc_message.encode(self.secret, method, iterations=self.iterations, **kwargs)

    def _error_message(self, code, message):
        return self._reply('error_response', error_code=code, error_message=message)

    def _default_method(self, *args, **kwargs):
        return self._error_message(404, "Method not implemented")

    def dispatch(self, envelope):
        try:
            unpack_message = rpc_message.decode(self.secret, envelope, iterations=self.iterations)
        except InvalidSignatureException:
            return self._error_message(403, "Unauthorized, invalid passphrase")

        unpack_kwargs = unpack_message[1]

        if unpack_message[0] is None:
            return self._error_message(502, "Malformed request - missing method field")

        method = unpack_message[0]
        if self.funcs.has_key(method):
            reply_args = self.funcs[method](**unpack_kwargs)
            return self._reply(None, **reply_args)

        return self._default_method()

    def bind(self, function, name=None):
        if name is None:
            name = function.__name__
        self.funcs[name] = function

if __name__ == '__main__':
    def ping(**kwargs):
        return dict(hello=kwargs)

    server = RPCServer(('127.0.0.1', 5959), 'moocows')
    server.bind(ping)
    server.serve_forever()



#!/usr/bin/env python
"""
Copyright (c) 2012, 2013 TortoiseLabs LLC

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

This software is provided 'as is' and without any warranty, express or
implied. In no event shall the authors be liable for any damages arising
from the use of this software.
"""

from distutils.core import setup

setup(name='ediarpc',
      version='0.2.1',
      description='A JSON-based RPC calling framework which supports inline signatures derived from passphrases',
      long_description="""
A library which provides JSON-based RPC calls over TCP sockets with inline signatures to
prove authenticity between two trusted parties.

Provides a ``ServerProxy`` like object for abstracting the details of the RPC system behind
python.""",
      author='William Pitcock',
      author_email='nenolod@tortois.es',
      url='http://bitbucket.org/tortoiselabs/ediarpc',
      packages=['ediarpc'],
      classifiers=['License :: OSI Approved :: ISC License (ISCL)', 'Operating System :: POSIX',
                   'Topic :: System :: Systems Administration'])

